<?php
/*
Plugin Name: AccessibilityPlus
Plugin URI: 
Description: Improve pagespeed insight or lighthouse accessibility scores by fixing recommended issues.
Version: 1.0.5
Author: easywpstuff
Author URI: https://easywpstuff.com
License: GNU General Public License v2 or later
*/

// Load the SimpleHTMLDOM library
require_once('inc/simple_html_dom.php');

// include option

include_once('inc/options.php');


// Name: Form elements do not have associated labels
function easywpstuff_assesplus_check_input_labels($html) {
  if (empty($html)) {
    return $html;
  }

  $dom = lhb_str_get_html($html, false, true, 'UTF-8', false, PHP_EOL, ' ');

  $inputs = $dom->find('input');
	
  foreach ($inputs as $input) {

    $label = $input->prev_sibling();
    if ($label && $label->tag == 'label') {
      continue;
    }

    if (!$input->getAttribute('aria-label')) {
      $input->setAttribute('aria-label', 'hidden');
    }
  }

  return $dom->save();
}

// Links do not have a discernible name
function easywpstuff_assesplus_add_aria_labels_to_links($html) {
  $picture_html = str_replace('</img>', '', $html);
  $picture_html = str_replace('</source>', '', $picture_html);
  
  $dom = lhb_str_get_html($picture_html, false, true, 'UTF-8', false, PHP_EOL, ' ');
  
  $links = $dom->find('a');
  $headerLinks = $dom->find('header a');
  $headerLinks = array_filter($headerLinks, function($link) {
    return $link->find('i', 0) || ($link->find('span', 0) && trim($link->plaintext));
  });
  $links = array_diff($links, $headerLinks);

  foreach ($links as $link) {
	$span = $link->find('span',0);
	$div = $link->find('div',0);
	$spnimg = $link->find('img',0);
	if($span && !$div && !$spnimg && !$link->getattribute('aria-label')){
      $span_text = trim($span->plaintext);
      if($span_text){
        $link->setattribute('aria-label', $span_text);
      }
    }
	$spanimg = $link->find('span img',0);
	if ($spanimg && !$link->getattribute('aria-label') && ($img->getattribute('alt')=='' || $img->getattribute('title')=='') ) {
		$link->setattribute('aria-label', 'image');
	}
    if (!trim($link->plaintext) && !$link->getattribute('aria-label')) {
      $child_elements = $link->find('*');
      if (empty($child_elements)) {
        $link->setattribute('aria-label', 'link');
      } else {
        $icon = $link->find('i', 0);
        if ($icon) {
          $link->setattribute('aria-label', 'icon');
        } else {
          $img = $link->find('img', 0);
          if ($img && !$img->getattribute('alt')) {
            $link->setattribute('aria-label', 'image');
          }
          else if ($img && ($img->getattribute('alt')=='' || $img->getattribute('title')=='')) {
            $link->setattribute('aria-label', 'image');
          }
          $svg = $link->find('svg', 0);
          if ($svg) {
            $link->setattribute('aria-label', 'svg');
          }
		  if (!$svg && !$img) {
			  $link->setattribute('aria-label', 'link');
		  }
        }
      }
    }
  }

  return $dom->save();
}


//Buttons do not have an accessible name

function easywpstuff_assesplus_add_aria_labels_to_buttons($html) {

  $dom = lhb_str_get_html($html, false, true, 'UTF-8', false, PHP_EOL, ' ');

  $buttons = $dom->find('button');

  foreach ($buttons as $button) {
    if (!trim($button->plaintext) && !$button->getattribute('aria-label')) {
      $child_elements = $button->find('*');
      if (empty($child_elements)) {
        $button->setattribute('aria-label', 'button');
      } else {
        $button->setattribute('aria-label', 'button');
      }
    }
	  if (!$button->getattribute('aria-label')) {
    $svg = $button->find('svg',0);
    $path = $button->find('path',0);
    $span = $button->find('span',0);
    if($svg && $path){
      $button->setattribute('aria-label', 'button');
    }elseif($span){
      $span_text = trim($span->plaintext);
      if($span_text){
        $button->setattribute('aria-label', $span_text);
      }
    }
  }
  }
  return $dom->save();
}

//[user-scalable="no"] is used in the <meta name="viewport"> element or the [maximum-scale] attribute is less than 5.
function easywpstuff_assesplus_modify_viewport_meta_tag($html) {
	
  $dom = lhb_str_get_html($html, false, true, 'UTF-8', false, PHP_EOL, ' ');

  $viewport_meta_tag = $dom->find('meta[name=viewport]', 0);
  if ($viewport_meta_tag) {
    $content = $viewport_meta_tag->getAttribute('content');
	  
    $content = preg_replace('/user-scalable=no/', '', $content);
	$content = preg_replace('/user-scalable=0/', '', $content);

    if (preg_match('/maximum-scale=([^,\s]*)/', $content, $matches)) {

      $maximum_scale = (float) $matches[1];
      if ($maximum_scale < 2) {
        $content = preg_replace('/maximum-scale=[^,\s]*/', 'maximum-scale=2', $content);
      }
    } else {
      $content .= ', maximum-scale=2';
    }

    $viewport_meta_tag->setAttribute('content', $content);
  }
  return $dom->save();
}


function easywpstuff_assesplus_add_aria_labels_to_headers($html) {
    $dom = lhb_str_get_html($html, false, true, 'UTF-8', false, PHP_EOL, ' '); // Parse the HTML into a DOM object

    foreach($dom->find('header div') as $div) {
        if ($div->role == 'link' && !$div->hasAttribute('aria-label')) {
            $div->setAttribute('aria-label', 'link');
        }
        if ($div->role == 'button' && !$div->hasAttribute('aria-label')) {
            $div->setAttribute('aria-label', 'button');
        }
        if ($div->role == 'menuitem' && !$div->hasAttribute('aria-label')) {
            $div->setAttribute('aria-label', 'menuitem');
        }
    }

    $html = $dom->save(); // Save the modified HTML
    return $html; // Return the modified HTML
}

// <frame> or <iframe> elements do not have a title
function easywpstuff_assesplus_booster_add_to_iframes($html) {
    $dom = lhb_str_get_html($html, false, true, 'UTF-8', false, PHP_EOL, ' '); 

    foreach($dom->find('iframe') as $iframe) {
        if (!$iframe->hasAttribute('title')) {
            $iframe->setAttribute('title', 'iframe');
        }
    }

    $html = $dom->save(); 
    return $html;
}

//Links are not crawlable
function easywpstuff_assesplus_addHashToBlankLinks($html) {
	$dom = lhb_str_get_html($html, false, true, 'UTF-8', false, PHP_EOL, ' ');
    $blankLinks = $dom->find('a');
    foreach ($blankLinks as $link) {
        if (!$link->href || $link->href === '') {
            $link->setattribute('href', '#');
        }
    }
	$html = $dom->save(); 
    return $html;
}

//ARIA progressbar elements do not have accessible names.
function easywpstuff_assesplus_add_aria_label_to_progressbar($html) {
    // Use Simple HTML DOM to parse the HTML
    $dom = lhb_str_get_html($html, false, true, 'UTF-8', false, PHP_EOL, ' '); 
    // Find all elements with the role="progressbar" attribute
    foreach($dom->find('[role=progressbar]') as $element) {
        // Check if the element has aria-label, aria-labelledby or title attribute
        $has_label = $element->hasAttribute('aria-label') || 
                     $element->hasAttribute('aria-labelledby') || 
                     $element->hasAttribute('title');
        // If the element doesn't have a label, add aria-label="progressbar"
        if (!$has_label || (empty($element->getAttribute('aria-label')) && empty($element->getAttribute('aria-labelledby')) && empty($element->getAttribute('title')))) {
            $element->setAttribute('aria-label', 'progressbar');
        }
    }
    // Return the modified HTML
    $html = $dom->save(); 
    return $html;
}
//Image elements do not have [alt] attributes
function easywpstuff_assesplus_addAltToImg($html) {
    $dom = lhb_str_get_html($html, false, true, 'UTF-8', false, PHP_EOL, ' ');
    $imgs = $dom->find('img');
    foreach ($imgs as $img) {
        if (!$img->alt) {
            $alt = '';
            if ($img->title) {
                $alt = $img->title;
            } else {
                $src = $img->src;
                $alt = basename($src);
                $alt = preg_replace('/\d+/', '', $alt);
                $alt = preg_replace('/\.[^.]+$/', '', $alt);
                $alt = str_replace('-', ' ', $alt);
				$alt = str_replace('_', ' ', $alt);
            }
            $img->setattribute('alt', $alt);
        }
    }
    $html = $dom->save();
    return $html;
}

// buffer if option is enabled
function easywpstuff_assesplus_bstr_template_redirect() {
	$options = get_option( 'easywpstuff_assesplus_booster_options' );
	
	
	if ( isset( $options['easywpstuff_assesplus_booster_field_no_labels'] ) && $options['easywpstuff_assesplus_booster_field_no_labels'] == 1 ) {
	
	   ob_start( 'easywpstuff_assesplus_check_input_labels');
	   
	}
	
	if ( isset( $options['easywpstuff_assesplus_booster_field_no_discernible_name'] ) && $options['easywpstuff_assesplus_booster_field_no_discernible_name'] == 1 ) {
	
	ob_start( 'easywpstuff_assesplus_add_aria_labels_to_links');
		
	}
	
	if ( isset( $options['easywpstuff_assesplus_booster_field_no_accessible_name'] ) && $options['easywpstuff_assesplus_booster_field_no_accessible_name'] == 1 ) {
	
	ob_start( 'easywpstuff_assesplus_add_aria_labels_to_buttons');
		
	}
	
	if ( isset( $options['easywpstuff_assesplus_booster_field_viewport_meta_tag'] ) && $options['easywpstuff_assesplus_booster_field_viewport_meta_tag'] == 1 ) {
	
	ob_start( 'easywpstuff_assesplus_modify_viewport_meta_tag');
		
	}
	
	if ( isset( $options['easywpstuff_assesplus_booster_field_viewport_meta_tag'] ) && $options['easywpstuff_assesplus_booster_field_viewport_meta_tag'] == 1 ) {
	
	ob_start( 'easywpstuff_assesplus_add_aria_labels_to_headers');
		
	}
	
	if ( isset( $options['easywpstuff_assesplus_booster_field_iframe_title_tag'] ) && $options['easywpstuff_assesplus_booster_field_iframe_title_tag'] == 1 ) {
	
	ob_start( 'easywpstuff_assesplus_booster_add_to_iframes');
		
	}
	
	if ( isset( $options['easywpstuff_assesplus_add_aria_label_to_progressbar'] ) && $options['easywpstuff_assesplus_add_aria_label_to_progressbar'] == 1 ) {
	
	ob_start( 'easywpstuff_assesplus_add_aria_label_to_progressbar');
		
	}
	
	if ( isset( $options['easywpstuff_assesplus_links_not_crawlable'] ) && $options['easywpstuff_assesplus_links_not_crawlable'] == 1 ) {
	
	ob_start( 'easywpstuff_assesplus_addHashToBlankLinks');
		
	}
	
	if ( isset( $options['easywpstuff_assesplus_img_donot_alt'] ) && $options['easywpstuff_assesplus_img_donot_alt'] == 1 ) {
	
	ob_start( 'easywpstuff_assesplus_addAltToImg');
		
	}
		
}
add_action( 'template_redirect', 'easywpstuff_assesplus_bstr_template_redirect', 99 );

// run the function
//add_action( 'plugins_loaded', 'easywpstuff_assesplus_loader' );
function easywpstuff_assesplus_loader() {
	if ( ! is_admin() ) {
     add_action('setup_theme', 'easywpstuff_assesplus_bstr_template_redirect', 99);
	}
}

// added settings link
function easywpstuff_assesplus_bs_settings_link($links) {
  $settings_link = '<a href="' . admin_url( 'options-general.php?page=accessibilityplus' ) . '">' . __( 'Settings' ) . '</a>';
  array_unshift( $links, $settings_link );
  return $links;
}
add_filter( 'plugin_action_links_' . plugin_basename( __FILE__ ), 'easywpstuff_assesplus_bs_settings_link' );

// enqueue admin styles on option page
function easywpstuff_assesplus_admin_enqueue_plugin_styles( $hook ) {
  if ( 'settings_page_accessibilityplus' === $hook ) {
    wp_enqueue_style( 'lhbooster-styles', plugin_dir_url( __FILE__ ) . 'assets/style.css' );
  }
}
add_action( 'admin_enqueue_scripts', 'easywpstuff_assesplus_admin_enqueue_plugin_styles' );
